#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2022
#
#   History:
#
#   v1.0: 2022-02: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
#FixWH=2000   # now dinamic

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer, WrkWH):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (WrkWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
    # adjust mean values
    adjustmeanv (newImage, newBG)
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # take care of the existing mean in determining the theoretical mean
    meanadj=40
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    #info_msg ("values mean="+str(meanl))
    qobj = 127.5*meanadj/100
    qsrc = meanl*(100.0-meanadj)/100
    thmean = qobj+qsrc   
    #info_msg ("q_obj,q_src,target="+str(qobj)+","+str(qsrc)+","+str(thmean))
    # range for the loops
    minmean = thmean*1.01  
    maxmean = thmean*0.99  
    
    modified=False
    if meanl < minmean and thmean!=meanl:
        ratio = thmean/meanl*1.0 
        #info_msg("mean value found = "+str(meanl)+" ratio = "+str(ratio))
        loop=0
        while meanl < minmean and loop<10:  
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE, 8,
                                             (0.0,0.0,0.3,0.3*ratio, 0.7,0.7*ratio, 1.0,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
            loop+=1
        modified=True
        #info_msg("increased mean value = "+str(meanl))
    if meanl > maxmean and thmean!=meanl:
        ratio = thmean/meanl*1.0 
        #info_msg("mean value found = "+str(meanl))
        loop=0
        while meanl > maxmean and loop<10:  
            pdb.gimp_drawable_curves_spline (inLayer, HISTOGRAM_VALUE, 8,
                                             (0.0,0.0,0.3,0.3*ratio, 0.7,0.7*ratio, 1.0,1.0))
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.00, 1.0)
            loop+=1
        modified=True
        #info_msg("reduced mean value = "+str(meanl))

    return(inLayer)

############################################################################
# Main
#---------------------------------------------------------------------------

def Gouache (inImage, inLayer,
                  UserOutWH, EffectLook,
                  doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height    
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # create new image to work on
    WrkWH=2000+500*int(EffectLook)
    newImage = createWrkimage (inImage, inLayer, WrkWH)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    # Apply a Paint look
    # thru Segmentation  
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -fx_segment_watershed 7,1,0,0,0,50,50 ')
    Painted = newImage.layers[0]
    Painted.mode = LAYER_MODE_NORMAL
    Painted.opacity = 100.0
    Painted.name = "Preprocessed painted thru Segmentation"
    pdb.plug_in_unsharp_mask (newImage, Painted, 3.0, 3.0, 0)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Scale down
    pdb.gimp_image_scale (newImage, width*2/3, height*2/3)
    # Apply samj_BoxFiter_Map 3,3,3,1,0,0,2,127,16,0,0,0.2
    #       samj_BoxFiter_Map 1,1,1,1,1,2,3,127,16,0,0,0.2
    # samj_BoxFiter_Test 2,3,3,1,2
    Gouacheparms = '2,3,3,1,2'
    pdb.plug_in_gmic_qt(newImage, Painted, 1,2,
    '-v - -samj_BoxFiter_Test '+Gouacheparms+' ')
    Gouache = newImage.layers[0]
    Gouache.mode = LAYER_MODE_HSV_VALUE
    Gouache.opacity = 95.0  
    Gouache.name = "samj_BoxFiter_Map "+Gouacheparms
    Hb=width/100; Vb=height/100
    pdb.plug_in_gauss_iir2 (newImage, Gouache, Hb, Vb)
    # Scale back
    pdb.gimp_image_scale (newImage, width, height)
    
    #pdb.gimp_display_new (newImage)
    #return

    # Add Source on top posterized
    pdb.gimp_image_set_active_layer (newImage, Painted)
    # Apply samj_Posterize_B 1,12,1,0,40,30,1,0,0,1,0,3,0,0,0,0,1,0
    #       samj_Posterize_B 5,32,1,8,400,100,10,0,1,0,0,1,0,0,0,0,0,0
    #       samj_Posterize_B 3,24,5,8,100,100,1,0,0,0,0,1,0,0,0,0,0,0
    Posterparms = '3,32,5,8,100,100,1,0,1,0,0,1,0,0,0,0,0,0'
    pdb.plug_in_gmic_qt(newImage, Painted, 1,2,
    '-v - -samj_Posterize_B  '+Posterparms+' ')
    Poster = newImage.layers[0]
    Poster.mode = LAYER_MODE_MULTIPLY
    Poster.opacity = 50.0  
    Poster.name = "samj_Posterize_B "+Posterparms
    
    # Add contours
    pdb.gimp_image_set_active_layer (newImage, Painted)
    # Apply samj_Posterize_B 1,12,1,0,40,30,1,0,0,1,0,3,0,0,0,0,1,0
    #       samj_Posterize_B 5,32,1,8,400,100,10,0,1,0,0,1,0,0,0,0,1,0
    PosterContoursparms = '3,32,5,8,100,100,1,0,1,1,0,1,0,0,0,0,1,0'
    pdb.plug_in_gmic_qt(newImage, Painted, 1,2,
    '-v - -samj_Posterize_B  '+PosterContoursparms+' ')
    PosterContours = newImage.layers[0]
    PosterContours.mode = LAYER_MODE_OVERLAY
    PosterContours.opacity = 20.0  
    PosterContours.name = "samj_Posterize_B "+PosterContoursparms
    Hb=width/30; Vb=height/30
    pdb.plug_in_spread (newImage, PosterContours, Hb, Vb)
    pdb.plug_in_gauss_iir2 (newImage, PosterContours, Hb, Vb)
    pdb.plug_in_spread (newImage, PosterContours, Hb/2, Vb/2)
    
    PaintedOnTop = Painted.copy()
    newImage.add_layer (PaintedOnTop, 0)
    pdb.gimp_drawable_desaturate (PaintedOnTop, DESATURATE_VALUE)
    PaintedOnTop.mode = LAYER_MODE_LINEAR_LIGHT
    PaintedOnTop.opacity = 88.0
    mask = pdb.gimp_layer_create_mask (PaintedOnTop, ADD_MASK_COPY)
    pdb.gimp_layer_add_mask (PaintedOnTop, mask)
    pdb.plug_in_vinvert (newImage, mask)
    pdb.gimp_drawable_curves_spline (mask, HISTOGRAM_VALUE,
                                     4, (0.0,0.33,1.0,1.0))
    PaintedOnTopC = Painted.copy()
    newImage.add_layer (PaintedOnTopC, 0)
    PaintedOnTopC.mode = LAYER_MODE_HSL_COLOR
    PaintedOnTopC.opacity = 50.0
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+" with a gouache painting look "
    effectLs = ["basic", "regular", "fine" ]
    effectLused = " (effect look "+effectLs[EffectLook]+")"
    outcome.name = outcome.name+effectLused
    
    # user required output image size
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != WrkWH:
        uRatio = UserOutWH*1.00/WrkWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
    
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2022"
desc="Gouache"

register(
    "Gouache",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   3000, (0, 7200, 100)),
        (PF_OPTION, "effect", "Effect look", 1, ["Basic", "Regular", "Fine" ]),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    Gouache,
    menu=menu
)

main()
